exception Halt of string

datatype typ = TClass of string

datatype expr =
    EInt of int
  | ENil
  | ESelf
  | EString of string
  | ELocRd of string    (* Read a local variable *)
  | ELocWr of string * expr  (* Write a local variable *)
  | EFldRd of string    (* Read a field *)
  | EFldWr of string * expr  (* Write a field *)
  | EIf of expr * expr * expr
  | ESeq of expr * expr
  | ENew of string
  | EInvoke of expr * string * expr list
  | ECast of expr * typ

type mtyp = (typ list) * typ

(*  meth name * arg name list * method body *)
type meth = { meth_name : string,
        meth_ret: typ,
	      meth_args : (string * typ) list,
        meth_locals : (string * typ) list,
	      meth_body : expr }

(* class name * superclass name * methods *)
type cls = { cls_name : string,
	     cls_super : string,
       cls_fields : (string * typ) list,
	     cls_meths : meth list }

(* classes * top-level expression *)
type prog = { prog_clss : cls list,
	           prog_main : expr }
