open Ast

fun print_expr (EInt i) = print (Int.toString i)
  | print_expr (ENil) = print "nil"
  | print_expr (ESelf) = print "self"
  | print_expr (EString s) = print ("\"" ^ s ^ "\"")
  | print_expr (ELocRd x) = print x
  | print_expr (ELocWr(x, e)) =
    (print x; print " = ("; print_expr e; print ")")
  | print_expr (EFldRd x) = print x
  | print_expr (EFldWr(x, e)) =
    (print x; print " = ("; print_expr e; print ")")
  | print_expr (EIf(e1, e2, e3)) =
    (print "if "; print_expr e1; print " then "; print_expr e2; print " else "; print_expr e3; print " end")
  | print_expr (ESeq(e1, e2)) =
    (print_expr e1; print "; "; print_expr e2)
  | print_expr (ENew x) =
    (print "new "; print x)
  | print_expr (EInvoke(e, m, es)) =
    (print_expr e; print "."; print m; print "("; print_exprs es; print ")")
  | print_expr (ECast(e, TClass t)) =
    (print "("; print_expr e; print ":"; print t; print ")")

and print_exprs [] = ()
  | print_exprs [e] = print_expr e
  | print_exprs (e::es) =
    (print_expr e; print ", "; print_exprs es)

and print_args [] = ()
  | print_args [(arg,TClass t)] = (print arg; print ":"; print t)
  | print_args ((arg,TClass t)::args) =
    (print arg; print ":"; print t; print ", "; print_args args)

and print_meth {meth_name=name, meth_ret=TClass ret, meth_args=args, meth_locals=locals, meth_body=body} =
    (print "  def ";
     print ret;
     print " ";
     print name;
     print "(";
     print_args args;
     print ")\n     ";
     print_args locals;
     print "\nbegin\n";
     print_expr body;
     print "\n  end")

and print_meths [] = ()
  | print_meths [m] = print_meth m
  | print_meths (m::ms) =
    (print_meth m; print "\n"; print_meths ms)

and print_cls {cls_name=name, cls_super=super, cls_fields=fields, cls_meths=meths} =
    (print "class "; print name; print " < "; print super; print "\n begin\n "; print_args fields; print "\n"; print_meths meths; print "\nend\n")

and print_clss [] = ()
  | print_clss [c] = print_cls c
  | print_clss (c::cs) = (print_cls c; print "\n"; print_clss cs)

and print_prog {prog_clss=clss, prog_main=main} = case clss of
						      [] =>  (print_expr main; print "\n")
						   |  _ => (print_clss clss; print "\n"; print_expr main; print "\n")
